from pathlib import Path
from .datatypes import (
    CStruct,
    CEnum,
    CEnumVal,
    CFunc,
    CHeader,
    CSource,
    CArg,
    CType,
    CPointer,
    CPointerType,
    CQualifier,
    CInclude,
)


def test_codegen():
    struct = CStruct(
        name="Str8",
        cargs=[
            CArg(name="size", _type=CType.U64),
            CArg(name="capacity", _type=CType.U64),
            CArg(name="buf", _type=CType.U8, pointer=CPointer(_type=CPointerType.SINGLE)),
        ],
    )

    cenum = CEnum(
        name="OS",
        values=[
            CEnumVal(name="OS_LINUX"),
            CEnumVal(name="OS_WINDOWS"),
            CEnumVal(name="OS_MACOS"),
        ]
    )

    typed_enum = CEnum(
        name="Compiler",
        values=[
            CEnumVal(name="COMPILER_GCC"),
            CEnumVal(name="COMPILER_CLANG"),
            CEnumVal(name="COMPILER_MSVC"),
        ],
        typedef=True,
    )

    main_func = CFunc(
        name="my_custom_func",
        ret_type=CType.I32,
        args=[
            CArg(name="argc", _type=CType.I32),
            CArg(name="argv", _type=CType.CHAR, pointer=CPointer(_type=CPointerType.DOUBLE), qualifier=CQualifier.CONST),
        ],
        body="  return 0;"
    )

    header = CHeader(
        name="str.h",
        includes=[
            CInclude(header="aliases.h", local=True),
        ],
        types=[struct, cenum, typed_enum],
        funcs=[main_func],
    )

    source = CSource(
        name="str.c",
        includes=[
            CInclude(header=header, local=True),
            CInclude(header="aliases.h", local=True),
        ],
        types=[],
        internal_funcs=[],
        funcs=[main_func]
    )

    header.save(Path("."))
    source.save(Path("."))


if __name__ == "__main__":
    test_codegen()
